/*
 * This file is a part of morm
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2018, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "dbexpression.h"


namespace morm
{


DbExpression::DbExpression()
{
	output_type = 0;
}


DbExpression::~DbExpression()
{
}


void DbExpression::set_output_type(int output_type)
{
	this->output_type = output_type;
}


bool DbExpression::can_field_be_generated(bool insertable, bool updatable, bool is_primary_key)
{
	if( output_type == MORM_OUTPUT_TYPE_DB_INSERT )
	{
		return insertable;
	}
	else
	if(	output_type == MORM_OUTPUT_TYPE_DB_UPDATE )
	{
		return updatable;
	}
	else
	if( output_type == MORM_OUTPUT_TYPE_DB_PRIMARY_KEY )
	{
		return is_primary_key;
	}
	else
	{
		return true;
	}
}



void DbExpression::field_before()
{
	BaseExpression::field_before();

	if( !is_first_field )
	{
		if( output_type == MORM_OUTPUT_TYPE_DB_INSERT ||
			output_type == MORM_OUTPUT_TYPE_DB_UPDATE )
		{
			(*out_stream) << ",";
		}
		else
		if( output_type == MORM_OUTPUT_TYPE_DB_PRIMARY_KEY )
		{
			(*out_stream) << " AND ";
		}
	}
}


void DbExpression::before_field_value_string()
{
	if( output_type == MORM_OUTPUT_TYPE_DB_INSERT ||
		output_type == MORM_OUTPUT_TYPE_DB_UPDATE ||
		output_type == MORM_OUTPUT_TYPE_DB_PRIMARY_KEY )
	{
		(*out_stream) << "'";
	}
}

void DbExpression::after_field_value_string()
{
	if( output_type == MORM_OUTPUT_TYPE_DB_INSERT ||
		output_type == MORM_OUTPUT_TYPE_DB_UPDATE ||
		output_type == MORM_OUTPUT_TYPE_DB_PRIMARY_KEY )
	{
		(*out_stream) << "'";
	}
}


void DbExpression::before_field_value(const std::wstring &)
{
	before_field_value_string();
}

void DbExpression::before_field_value(const std::string &)
{
	before_field_value_string();
}

void DbExpression::after_field_value(const std::wstring &)
{
	after_field_value_string();
}

void DbExpression::after_field_value(const std::string &)
{
	after_field_value_string();
}




}
