/*
 * This file is a part of morm
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2018, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef headerfile_morm_dbexpression
#define headerfile_morm_dbexpression

#include "baseexpression.h"
#include "morm_types.h"


namespace morm
{

class DbExpression : public BaseExpression
{
public:

	DbExpression();
	virtual ~DbExpression();

	virtual void set_output_type(int output_type);



protected:

	int output_type;



	bool can_field_be_generated(bool insertable, bool updatable, bool is_primary_key);

	void field_before();
	void before_field_value(const std::wstring &);
	void before_field_value(const std::string &);
	void after_field_value(const std::wstring &);
	void after_field_value(const std::string &);



	template<typename FieldValue>
	void print_field_name_value(const wchar_t * field_name, const FieldValue & field_value)
	{
		put_field_name(field_name);

		if( output_type == MORM_OUTPUT_TYPE_DB_PRIMARY_KEY )
		{
			(*out_stream) << " = ";
		}

		put_field_value(field_value);
	}


private:

	void before_field_value_string();
	void after_field_value_string();





//	DbExpression & eq(const wchar_t * field_name, const std::wstring & field_value)
//	{
//		//put_field_name(field_name);
//		return *this;
//	}
//
//	DbExpression & eq(const wchar_t * field_name, const std::string & field_value)
//	{
//
//		return *this;
//	}
//
//	template<typename FieldValue>
//	DbExpression & eq(const wchar_t * field_name, const FieldValue & field_value)
//	{
//		return *this;
//	}




};

}

#endif
