/*
 * This file is a part of morm
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2018, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef headerfile_morm_postgresqlconnector
#define headerfile_morm_postgresqlconnector

#include "dbconnector.h"
#include <libpq-fe.h>
#include <string>



namespace morm
{

class PostgreSQLConnector : public DbConnector
{
public:
	PostgreSQLConnector();
	virtual ~PostgreSQLConnector();

	void set_log_queries(bool log_queries);

	bool query(const char * query_str);
	bool query_select(const char * query_str);
	bool query_update(const char * query_str);
	bool query_insert(const char * query_str);

	bool is_last_result(ExecStatusType t);

	/*
	 * get column index from the last query (select)
	 * returns -1 if there is no such a column
	 *
	 */
	int get_column_index(const char * column_name);

	bool is_null(int row, int col);

	/*
	 * return an item from the last query
	 * can return a null pointer if there is no such an item in the last result
	 *
	 */
	const char * get_value(int row, int col);

	bool get_value(int row, int col, std::string & result, bool clear_string = true);
	bool get_value(int row, int col, std::wstring & result, bool clear_string = true);
	void get_value_bin(int row, int col, std::string & result, bool clear_string = true);

	void clear_result();

	//void SetConnParam(const std::wstring & database, const std::wstring & user, const std::wstring & pass);
	//void Connect();
	//void WaitForConnection();
	//void Close();
	//bool AssertConnection(bool put_log = true, bool throw_if_no_connection = true);
	//void SetDbParameters();
	//PGconn * GetPgConn();


protected:

	PGconn * pg_conn;
	PGresult * last_result; // can be null
	ExecStatusType last_status;
	bool log_queries;

	std::wstring db_database;
	std::wstring db_user;
	std::wstring db_pass;


	void allocate_default_expression();

};

}

#endif
