/*
 * This file is a part of morm
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2018, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "postgresqlexpression.h"


namespace morm
{


void PostgreSQLExpression::before_field_value_string()
{
	if( output_type == MORM_OUTPUT_TYPE_DB_INSERT ||
		output_type == MORM_OUTPUT_TYPE_DB_UPDATE ||
		output_type == MORM_OUTPUT_TYPE_DB_PRIMARY_KEY )
	{
		(*out_stream) << "E'";
	}
}

void PostgreSQLExpression::after_field_value_string()
{
	if( output_type == MORM_OUTPUT_TYPE_DB_INSERT ||
		output_type == MORM_OUTPUT_TYPE_DB_UPDATE ||
		output_type == MORM_OUTPUT_TYPE_DB_PRIMARY_KEY )
	{
		(*out_stream) << "'";
	}
}


void PostgreSQLExpression::before_field_value(const std::wstring &)
{
	before_field_value_string();
}

void PostgreSQLExpression::before_field_value(const std::string &)
{
	before_field_value_string();
}

void PostgreSQLExpression::after_field_value(const std::wstring &)
{
	after_field_value_string();
}

void PostgreSQLExpression::after_field_value(const std::string &)
{
	after_field_value_string();
}

void PostgreSQLExpression::put_name_value_separator()
{
	(*out_stream) << '=';
}


void PostgreSQLExpression::esc(char val, PT::TextStream & stream)
{
	switch( val )
	{
	case '\\': stream << "\\\\"; break;
	case '\'': stream << "\\\'"; break; // don't use "''" because we use the method for PQconnectdb too
	default:
		if( val != 0 )
		{
			stream << val;
		}
	}
}




}
